%{
#include <stdio.h>
#include <stdlib.h>

#include "lexer.h"

#undef stderr
#define stderr stdout

int lineno;

extern char     state, charset;
extern int	debug;
extern int	yyinput(char *buf, size_t size);
#define YY_INPUT(buf,result,max_size) result = yyinput(buf, max_size)
%}

%option debug nodebug
%option never-interactive 
%option noyywrap

TOKENFRONT	[^[:blank:][:cntrl:][:digit:][:punct:]]
TOKENMID	[^[:blank:]<>;=():&%$#@+|/\\{}^\"\?\*,[:cntrl:]\[\]]+
TOKENBACK	[^[:blank:]<>;=():&%$#@+|/\\{}^\"\?\*\._\-\+,\[\][:cntrl:]]

TOKEN		{TOKENFRONT}{TOKENMID}{TOKENBACK}{1,70}

BCHARSNOSPC	[0-9a-zA-Z'()+_,-./:=?#]
BCHARS		[0-9a-zA-Z'()+_,-./:=?# ]
MIME_BOUNDARY	{BCHARS}{0,69}{BCHARSNOSPC}

CHARSET		[0-9a-zA-Z-]+
ENCODED_WORD	=\?{CHARSET}\?[bq]\?[^\?]*\?\=
ENCODED_TOKEN	({ENCODED_WORD}[ \t\n]+)*{ENCODED_WORD}

%s TEXT COMMENT

%%
<INITIAL>^To/:.*			{ if (state=='h') set_tag("to:"  ); }
<INITIAL>^From/:.*			{ if (state=='h') set_tag("from:"); }
<INITIAL>^Return-Path/:			{ if (state=='h') set_tag("rtrn:"); }
<INITIAL>^Subject/:			{ if (state=='h') set_tag("subj:"); }
<INITIAL>^Date:.*			{ }

<INITIAL>{ENCODED_TOKEN}			{ return TOKEN; }
<INITIAL>\n[ \t]				{ lineno += 1; }
<INITIAL>^\n					{ BEGIN TEXT;
						  state='b'; if (debug) printf("*** state='%c'\n", state);
						  return(EMPTY);
						}

^--mime_boundary				{ BEGIN INITIAL;
						  state='h'; if (debug) printf("*** state='%c'\n", state);
						  return BOUNDARY;
						}

<TEXT>"<!"					{ BEGIN COMMENT; }
<COMMENT>{TOKEN}				{ /* discard innards of html tokens and comments */ }
<COMMENT>"-->"					{ BEGIN TEXT; }

base64						{ charset='b'; if (debug) printf("BASE64\n"); return TOKEN; }
qp						{ charset='q'; if (debug) printf("QP\n"); return TOKEN; }

{TOKEN}						{ return TOKEN; }

.						;			/* ignore character */
\n						{ lineno += 1;
						  clr_tag();
	/*
						  if (state=='h') {
						    state='b'; if (debug) printf("*** state='%c'\n", state);
						  }
	*/
						}

%%

void lexer_set_debug(int v);
void lexer_set_debug(int v)
{
#ifdef	FLEX_DEBUG
    yy_flex_debug = v;
#endif
}
