#include <stdio.h>
#include <stdlib.h>

#include "lexer.h"

/* External Variables */
extern char *yytext;
extern int   yyleng;

/* Function Prototypes */
extern int yylex();
extern void lexer_set_debug(int v);

/* Global Variables */
int debug = 0;
char state = 'h';
char charset = ' ';
const char *tag = NULL;

/* Function Definitions */

int yyinput(char *buf, size_t max_size)
{
    int count;
    static int mylineno = 0;

    if (fgets(buf, max_size, stdin) == NULL)
	return 0;

    count = strlen(buf);

    if (debug > 0)
	fprintf(stdout, "*** %2d %c%c %2d %s", ++mylineno, state, charset, count, buf);

    if (state == 'b' && count > 1 && memcmp(buf, "--", 2) != 0) {
	int cnt = count;
	switch (charset) {
	case 'b':
	    cnt = base64_decode(buf, count);
	    break;
	case 'q':
	    cnt = qp_decode(buf, count);
	    break;
	}
	if (debug > 0 && cnt != count)
	    fprintf(stdout, "*** %2d %c%c %2d %s", mylineno, state, charset, count, buf);
	count = cnt;
    }

    return count;
}

void got_mime_boundary(void)
{
    char *m;
    if (yytext[yyleng-1] != '-') {
	m = "begin";
    }
    else {
	m = "end";
    }
//    fprintf(stdout, "fcn:         %s - %s\n", "boundary", m);
}

void mime_boundary_set(void)
{
//  fprintf(stdout, "fcn:         %s\n", "mime_boundary_set");
}

void clr_tag(void)
{
    tag = NULL;
    if (debug > 1)
    fprintf(stdout, "***         tag clr\n");
}

void set_tag(const char *s)
{
    tag = s;
    if (debug > 1)
	fprintf(stdout, "***         tag set - %s\n", tag);
}

int main(int argc, char **argv)
{
    int token;

    while (argc-- > 1) {
	char *arg = *++argv;
	switch (arg[1]) {
	case 'd': 
	    debug += 1;
	    break;
	}
    }

    lexer_set_debug(debug);

    while ((token = yylex()) > NONE) {
	char *s;
	for (s = yytext; *s; s +=1) {
	    if (*s == '\n')
		*s = ' ';
	}
	if (token == FROM)
	    state = 'h';
	if (token == EMPTY)
	    continue;
	if (token == BOUNDARY)
	    continue;
	if (debug > 2)
	    fprintf(stdout, "tok: %2d      ", token);
	fprintf(stdout, "%s%s\n", tag == NULL ? "" : tag, yytext);
    }

    return 0;
}
