/* $Id: qp.c,v 1.6 2003/09/02 16:14:47 relson Exp $ */

/*****************************************************************************

NAME:
   qp.c -- decode quoted printable text

AUTHOR:
   David Relson <relson@osagesoftware.com>

******************************************************************************/

#include "qp.h"

/* Local Variables */

static char qp_xlate[256];

static int hex_to_bin(char c) {
    switch(c) {
	case '0': return 0;
	case '1': return 1;
	case '2': return 2;
	case '3': return 3;
	case '4': return 4;
	case '5': return 5;
	case '6': return 6;
	case '7': return 7;
	case '8': return 8;
	case '9': return 9;
	case 'a': case 'A': return 10;
	case 'b': case 'B': return 11;
	case 'c': case 'C': return 12;
	case 'd': case 'D': return 13;
	case 'e': case 'E': return 14;
	case 'f': case 'F': return 15;
	default: return -1;
    }
}

/* Function Definitions  */

size_t qp_decode(char *text, size_t leng)
{
    size_t size = leng;
    char *s = text;	/* src */
    char *d = text;	/* dst */
    char *e = s + size;		/* end */

    while (s < e)
    {
	char ch = *s++;
	int x, y;
	if (ch == '=' && s <= e && s[0] == '\n') {
	    /* continuation line, trailing = */
	    s++;
	    continue;
	}
	if (ch == '=' && s + 2 <= e && 
	    (y = hex_to_bin(s[0])) >= 0 && (x = hex_to_bin(s[1])) >= 0) {
	    /* encoded character */
	    ch = y << 4 | x;
	    s += 2;
	}
	*d++ = ch;
    }
    *d = '\0';
    return d - text;
}

/* rfc2047 - QP    [!->@-~]+
*/

static void qp_init(void)
{
    size_t i;
    static int first = 1;

    if (!first)
	return;
    first = 0;

    for (i = '!'; i <= '~'; i += 1) {
	qp_xlate[i] = (char) i;
    }

    qp_xlate['?'] = 0;
    qp_xlate[' '] = ' ';
    qp_xlate['_'] = ' ';

    return;
}

int qp_validate(char *text, size_t leng)
{
    size_t i;

    qp_init();

    for (i = 0; i < leng; i += 1) {
	char b = text[i];
	char v = qp_xlate[b];
	if (v == 0)
	    return 0;
	else
	    text[i] = v;
    }

    return 1;
}
